@@ -99,36 +99,33 @@ def cli(ctx, project_file, verbose, project_id, region_id, account_id, role_arn,
     project = projects.get(project_id)
     project['id'] = project_id
 
-    if verbose and project:
-        click.echo(f"Loaded {project_file} {project}")
-
     if role_arn:
         project['role_arn'] = role_arn
-        if verbose:
-            click.echo(f"Using role_arn {project['role_arn']}")
 
     if region_id:
         project['aws_region_name'] = region_id
-        if verbose:
-            click.echo(f"Using aws_region_name {project['aws_region_name']}")
 
     user_details = Iam(role_arn)
     caller_identity = user_details.caller_identity()
     underlying_caller_identity = user_details.caller_identity(underlying=True)
 
-    if verbose:
-        click.echo(f"Running as role {caller_identity['arn']}")
-        if caller_identity['arn'] != underlying_caller_identity['arn']:
-            click.echo(f"Underlying role {underlying_caller_identity['arn']}")
-        click.echo("")
-
     if account_id:
         project['account_id'] = account_id
     else:
         project['account_id'] = caller_identity['account_id']
 
     if verbose:
-        click.echo(f"Using account_id {project['account_id']}")
+        click.echo(click.style(f"Loaded {project_file}:", fg="cyan"))
+        pprint(project)
+        click.echo("")
+        click.echo(click.style(f"Using role_arn:         {project['role_arn']}", fg="cyan"))
+        click.echo(click.style(f"Using aws_region_name:  {project['aws_region_name']}", fg="cyan"))
+        click.echo(click.style(f"Running as role:        {caller_identity['arn']}", fg="cyan"))
+        if caller_identity['arn'] != underlying_caller_identity['arn']:
+            click.echo(click.style(f"Underlying role:        {underlying_caller_identity['arn']}", fg="cyan"))
+
+        click.echo(click.style(f"Using account_id:       {project['account_id']}", fg="cyan"))
+        click.echo("")
 
     ctx.obj = {
         'project_filepath': project_file,
@@ -154,55 +151,68 @@ def publish(ctx, service_id, namespace, label):
     account_id = project['account_id']
     region_id = project['aws_region_name']
 
-    ecr = Ecr(account_id, role_arn)
+    ecr = Ecr(account_id, region_id, role_arn)
     parameter_store = SsmParameterStore(project['id'], role_arn)
 
-    print(f"*** Attempting to publish {project['id']}/{service_id}")
+    click.echo(click.style(f"Attempting to publish {project['id']}/{service_id}", fg="blue"))
 
     profile_name = None
     if role_arn:
         profile_name = 'service_publisher'
         configure_aws_profile(role_arn, profile_name)
 
+    click.echo(click.style(f"Authenticating {account_id} for `docker push` with ECR", fg="yellow"))
+
     ecr.login(profile_name)
 
-    remote_image_name = ecr.publish_image(
+    remote_image_name, remote_image_tag, local_image_tag = ecr.publish_image(
         namespace,
         service_id,
-        label,
-        region_id,
         dry_run
     )
 
-    parameter_store.update_ssm(
+    click.echo(click.style(f"Published Image {local_image_tag} to {remote_image_name}", fg="yellow"))
+
+    ecr.retag_image(
+        namespace,
+        service_id,
+        remote_image_tag,
+        label
+    )
+
+    click.echo(click.style(f"Tagged {remote_image_name} to {remote_image_name}", fg="yellow"))
+
+    ssm_path = parameter_store.update_ssm(
         service_id,
         label,
         remote_image_name,
         dry_run
     )
 
-    print(f"*** Done publishing {project['id']}/{service_id}")
+    click.echo(click.style(f"Updated SSM path {ssm_path} to {remote_image_name}", fg="yellow"))
+
+    click.echo(click.style(f"Done publishing {project['id']}/{service_id}", fg="green"))
 
 
 @cli.command()
-@click.option('--project-id', '-i', prompt="Enter an id for this project", help="The project ID")
 @click.option('--project-name', '-n', prompt="Enter a descriptive name for this project",
               help="The name of the project")
 @click.option('--environment-id', '-e', prompt="Enter an id for an environment", help="The primary environment's ID")
 @click.option('--environment-name', '-a', prompt="Enter a descriptive name for this environment",
               help="The primary environment's name")
 @click.pass_context
-def initialise(ctx, project_id, project_name, environment_id, environment_name):
+def initialise(ctx, project_name, environment_id, environment_name):
     role_arn = ctx.obj['role_arn']
     verbose = ctx.obj['verbose']
     dry_run = ctx.obj['dry_run']
+    project = ctx.obj['project']
 
     project_filepath = ctx.obj['project_filepath']
 
-    releases_store = DynamoDbReleaseStore(project_id, role_arn)
+    releases_store = DynamoDbReleaseStore(project["id"], role_arn)
 
     project = {
-        'id': project_id,
+        'id': project["id"],
         'name': project_name,
         'role_arn': role_arn,
         'environments': [
@@ -233,17 +243,24 @@ def initialise(ctx, project_id, project_name, environment_id, environment_name):
 @click.option('--release-id', prompt="Release ID to deploy", default="latest", show_default=True,
               help="The ID of the release to be deployed, or the latest release if unspecified")
 @click.option('--environment-id', prompt="Environment ID to deploy release to",
+              default="stage", show_default=True,
               help="The target environment of this deployment")
+@click.option("--namespace", default=DEFAULT_ECR_NAMESPACE, show_default=True)
 @click.option('--description', prompt="Enter a description for this deployment",
-              help="A description of this deployment", default="No description given.")
+              help="A description of this deployment", default="No description provided")
 @click.pass_context
-def deploy(ctx, release_id, environment_id, description):
+def deploy(ctx, release_id, environment_id, namespace, description):
     project = ctx.obj['project']
     role_arn = ctx.obj['role_arn']
     dry_run = ctx.obj['dry_run']
 
     releases_store = DynamoDbReleaseStore(project['id'], role_arn)
     parameter_store = SsmParameterStore(project['id'], role_arn)
+    account_id = project['account_id']
+    region_id = project['aws_region_name']
+
+    ecr = Ecr(account_id, region_id, role_arn)
+
     user_details = Iam(role_arn)
 
     environments = get_environments_lookup(project)
@@ -258,22 +275,64 @@ def deploy(ctx, release_id, environment_id, description):
     else:
         release = releases_store.get_release(release_id)
 
-    click.echo(click.style("Release to deploy:", fg="blue"))
-    click.echo(pprint(release))
-    click.confirm(click.style("create deployment?", fg="green", bold=True), abort=True)
+    click.echo("")
+    click.echo(click.style(f"Deploying release {release['release_id']}", fg="blue"))
+    click.echo(click.style(f"Requested by: {release['requested_by']}", fg="yellow"))
+    click.echo(click.style(f"Date created: {release['date_created']}", fg="yellow"))
+
+    click.echo("")
+    for service, image in release['images'].items():
+        click.echo(click.style(f"{service}: {image}", fg="bright_yellow"))
+
+    click.echo("")
+    click.confirm(click.style("Create deployment?", fg="green", bold=True), abort=True)
 
     caller_identity = user_details.caller_identity(underlying=True)
     deployment = create_deployment(environment, caller_identity['arn'], description)
 
-    click.echo(click.style("Created deployment:", fg="blue"))
-    click.echo(pprint(deployment))
+    click.echo("")
+    click.echo(click.style("Created deployment.", fg="green"))
+    click.echo("")
+    click.echo(click.style(f"Requested by: {deployment['requested_by']}", fg="yellow"))
+    click.echo(click.style(f"Date created: {deployment['date_created']}", fg="yellow"))
 
-    if not dry_run:
-        releases_store.add_deployment(release['release_id'], deployment)
-        parameter_store.put_services_to_images(environment_id, release['images'])
-    else:
+    releases_store.add_deployment(
+        release_id=release['release_id'],
+        deployment=deployment,
+        dry_run=dry_run
+    )
+
+    for service_id, image_name in release['images'].items():
+        ssm_path = parameter_store.update_ssm(
+            service_id=service_id,
+            label=environment_id,
+            image_name=image_name,
+            dry_run=dry_run
+        )
+
+        click.echo("")
+        click.echo(click.style(f"*** {service_id}: Updated SSM path {ssm_path} to {image_name}", fg="yellow"))
+
+        old_tag = image_name.split(":")[-1]
+        new_tag = f"env.{environment_id}"
+
+        ecr.retag_image(
+            namespace=namespace,
+            service_id=service_id,
+            tag=old_tag,
+            new_tag=new_tag,
+            dry_run=dry_run
+        )
+
+        click.echo(click.style(
+            f"*** {service_id}: Retagged image {service_id}:{old_tag} to {service_id}:{new_tag}", fg="yellow")
+        )
+        click.echo("")
+
+    if dry_run:
         click.echo("dry-run, not created.")
-        return
+
+    click.echo(click.style(f"Deployed {service_id} to {new_tag}", fg="green"))
 
 
 @cli.command()
@@ -281,18 +340,34 @@ def deploy(ctx, release_id, environment_id, description):
               help="The existing label upon which this release will be based", default="latest", show_default=True)
 @click.option('--service-id', prompt="Service to update", default="all", show_default=True,
               help="The service to update with a (prompted for) new image")
-@click.option('--release-description', prompt="Description for this release", default="No description given.")
+@click.option('--release-description', prompt="Description for this release", default="No description provided")
 @click.pass_context
 def prepare(ctx, from_label, service_id, release_description):
     project = ctx.obj['project']
+    account_id = project['account_id']
+    region_id = project['aws_region_name']
     role_arn = ctx.obj['role_arn']
     dry_run = ctx.obj['dry_run']
 
+    ecr = Ecr(account_id, region_id, role_arn)
     releases_store = DynamoDbReleaseStore(project['id'], role_arn)
     parameter_store = SsmParameterStore(project['id'], role_arn)
     user_details = Iam(role_arn)
 
-    from_images = parameter_store.get_services_to_images(from_label)
+    image_repositories = project.get('image_repositories')
+
+    from_images = {}
+    if image_repositories:
+        for image in image_repositories:
+            service_id = image['id']
+            account_id = image.get('account_id', account_id)
+            namespace = image.get('namespace', DEFAULT_ECR_NAMESPACE)
+
+            image_details = ecr.describe_image(namespace, service_id, from_label, account_id)
+            from_images[image_details['service_id']] = image_details['ref']
+    else:
+        from_images = parameter_store.get_services_to_images(from_label)
+
     service_source = "latest"
     if service_id == "all":
         release_image = {}
@@ -301,7 +376,12 @@ def prepare(ctx, from_label, service_id, release_description):
         if _is_url(service_source):
             release_image = {service_id: service_source}
         else:
-            release_image = parameter_store.get_service_to_image(service_source, service_id)
+            if image_repositories:
+                image_details = ecr.describe_image(namespace, service_id, service_source, account_id)
+                release_image = {image_details['service_id']: image_details['ref']}
+            else:
+                release_image = parameter_store.get_service_to_image(service_source, service_id)
+
     release_images = {**from_images, **release_image}
 
     if not release_images:
@@ -316,11 +396,26 @@ def prepare(ctx, from_label, service_id, release_description):
         release_description,
         release_images)
 
+    click.echo("")
     if service_id == "all":
-        click.echo(f"Prepared release from images in {from_label}")
+        click.echo(click.style(
+            f"Prepared release from images in {from_label}", fg="blue")
+        )
     else:
-        click.echo(f"Prepared release from images in {from_label} with {service_id} from {service_source}")
-    click.echo(pprint(release))
+        click.echo(click.style(
+            f"Prepared release from images in {from_label} with {service_id} from {service_source}", fg="blue")
+        )
+
+    click.echo(click.style(f"Requested by: {release['requested_by']}", fg="yellow"))
+    click.echo(click.style(f"Date created: {release['date_created']}", fg="yellow"))
+    click.echo("")
+
+    for service, image in release['images'].items():
+        click.echo(click.style(f"{service}: {image}", fg="bright_yellow"))
+
+    click.echo("")
+
+    click.echo(click.style(f"Created release {release['release_id']}", fg="green"))
 
     if not dry_run:
         releases_store.put_release(release)
